#include "../csg.hpp"

#include "csg_types.hpp"

namespace csg {

class CsgIF::Impl {
public:
  double call_signed_distance(const std::shared_ptr<Tree> &a_tree, double xx,
                              double yy, double zz) const {
    return signed_distance(a_tree->top, xx, yy, zz);
  }
};

CsgIF::CsgIF(const CsgIF &rhs)
    : m_state(rhs.m_state), m_pimpl(std::make_unique<Impl>(*rhs.m_pimpl)) {}

CsgIF::CsgIF(std::shared_ptr<Tree> a_state)
    : m_state(a_state), m_pimpl(std::make_unique<Impl>()) {}

CsgIF::CsgIF(CsgIF &&rhs) noexcept = default;
CsgIF::~CsgIF() = default;

double CsgIF::operator()(double xx, double yy, double zz) const noexcept {
  return m_pimpl->call_signed_distance(m_state, xx, yy, zz);
}

} // namespace csg
