#include <iostream>

#include "solver.hpp"

namespace solver {

std::optional<solver::GeometrySettings> make_geometry(solver::InputInfo ii) {
  solver::GeometrySettings geo;
  if (!ii.count(PROB_LO)) {
    require(PROB_LO);
    return std::nullopt;
  }
  if (!ii.count(PROB_HI)) {
    require(PROB_HI);
    return std::nullopt;
  }
  if (!ii.count(PERIODIC)) {
    require(PERIODIC);
    return std::nullopt;
  }
  auto lows = std::get<solver::NumberArray>(ii[PROB_LO]);
  auto highs = std::get<solver::NumberArray>(ii[PROB_HI]);
  auto is_periodic = std::get<solver::NumberArray>(ii[PERIODIC]);
  if (lows.size() != 3) {
    std::cout << "prob_lo needs 3 elements " << std::endl;
    return std::nullopt;
  }
  if (highs.size() != 3) {
    std::cout << "prob_hi needs 3 elements " << std::endl;
    return std::nullopt;
  }
  if (is_periodic.size() != 3) {
    std::cout << "periodic needs 3 elements " << std::endl;
    return std::nullopt;
  }

  std::get<0>(geo.axes).high = highs[0];
  std::get<0>(geo.axes).low = lows[0];
  std::get<0>(geo.axes).periodic = is_periodic[0];
  std::get<1>(geo.axes).high = highs[1];
  std::get<1>(geo.axes).low = lows[1];
  std::get<1>(geo.axes).periodic = is_periodic[1];
  std::get<2>(geo.axes).high = highs[2];
  std::get<2>(geo.axes).low = lows[2];
  std::get<2>(geo.axes).periodic = is_periodic[2];
  return geo;
}
} // namespace solver
