#include <iostream>
#include <optional>
#include <sstream>

#include "solver.hpp"

namespace solver {

void require(std::string key) {
  std::cout << "missing required key:  " << key << std::endl;
}

std::optional<solver::SolverSettings> do_make_solver(solver::InputInfo ii) {
  solver::SolverSettings ss;
  auto new_geo = make_geometry(ii);
  if (!new_geo.has_value()) {
    std::cout << "Problem making geometry" << std::endl;
    return std::nullopt;
  }
  auto new_mesh = make_mesh(ii);
  if (!new_mesh.has_value()) {
    std::cout << "Problem making mesh" << std::endl;
    return std::nullopt;
  }
  auto new_time = make_time(ii);
  if (!new_time.has_value()) {
    std::cout << "Problem making time" << std::endl;
    return std::nullopt;
  }
  ss.geometry = new_geo.value();
  ss.time = new_time.value();
  ss.mesh = new_mesh.value();
  return ss;
}

std::optional<solver::SolverSettings> make_solver(solver::InputInfo ii) {
  auto maybe_state = do_make_solver(ii);
  if (!maybe_state.has_value()) {
    return std::nullopt;
  }
  return maybe_state.value();
}

std::string serialize(solver::SolverSettings settings) {

  std::stringstream buffer;

  auto [xx, yy, zz] = settings.geometry.axes;
  buffer << std::endl;
  buffer << "  geometry.is_periodic = " << xx.periodic << " " << yy.periodic
         << " " << zz.periodic << std::endl;
  buffer << "  geometry.prob_lo     = " << xx.low << " " << yy.low << " "
         << zz.low << std::endl;
  buffer << "  geometry.prob_hi     = " << xx.high << " " << yy.high << " "
         << zz.high << std::endl;

  auto [mx, my, mz] = settings.mesh.axes;
  buffer << "  amr.n_cell      = " << mx.n_cell << " " << my.n_cell << " "
         << mz.n_cell;
  return buffer.str();
}

} // namespace solver
